from copy import copy

import hpccm
from hpccm import linux_distro
from hpccm.building_blocks import (
    openmpi,
    generic_autotools,
    pmix,
    slurm_pmi2,
    ucx,
    gdrcopy,
    knem,
    multi_ofed,
    mlnx_ofed,
)
from hpccm.primitives import comment, copy, label, shell, environment, runscript

if hpccm.config.g_linux_distro != linux_distro.CENTOS:
    print(
        "OpenMPI is supported on CentOS only! Please supply --base_image 'centos:8' parameter!"
    )
    exit(1)

print(f"Evaluating {filename}")

# Add cli arguments to args_parser
parse_g = parser.add_argument_group(filename)
parse_g.add_argument(
    "--ompi",
    type=str,
    default="4.0.5",
    help="OpenMPI version, e.g. 2.1.1, 2.1.5, 3.0.1, 3.1.2",
)
parse_g.add_argument(
    "--mpi_benchmarks",
    dest="mpi_benchmarks",
    action="store_true",
    help="Installs OSU MPI " "benchmarks as scif app and mpi_bw, mpi_ring," "mpi_hello",
)
# parse_g.add_argument(
#     "--ucx",
#     type=str,
#     default="1.8.1",
#     help="UCX version",
# )
# parse_g.add_argument(
#     "--slurm",
#     type=str,
#     default="20.02.5",
#     help="Slurm version",
# )
# parse_g.add_argument(
#     "--infiniband",
#     dest="infiniband",
#     action="store_true",
#     help="Enable Infiniband",
# )

# Parse local args
local_args = parser.parse_known_args()[0]

# set image file name
img_file += f"-openmpi-{local_args.ompi}"

# Optionally set out_dir
out_dir += f"/openmpi/{local_args.ompi}"

# Implement recipe
Stage0 += comment(f"--- Begin {filename} ---")

# Begin copy from https://github.com/NVIDIA/hpc-container-maker/blob/master/recipes/osu_benchmarks/common.py
Stage0 += shell(commands=["ln -s /usr/bin/gcc /usr/bin/cc"])  # Fix for gdrcopy
Stage0 += gdrcopy(ldconfig=True)
Stage0 += knem(ldconfig=True)

# Mellanox legacy OFED support
mlnx_versions = ["4.6-1.0.1.1", "4.7-3.2.9.0", "4.9-0.1.7.0", "4.9-2.2.4.0"]
Stage0 += multi_ofed(
    inbox=False, mlnx_versions=mlnx_versions, prefix="/usr/local/ofed", symlink=False
)

# RDMA-core based OFED support
Stage0 += mlnx_ofed(version="5.1-0.6.6.0", symlink=False)

# UCX default - RDMA-core based OFED
Stage0 += ucx(
    version="1.9.0",
    cuda=False,
    gdrcopy="/usr/local/gdrcopy",
    knem="/usr/local/knem",
    disable_static=True,
    enable_mt=True,
)

# UCX - Mellanox legacy support
Stage0 += ucx(
    version="1.9.0",
    build_environment={
        "LD_LIBRARY_PATH": "/usr/local/ofed/4.6-1.0.1.1/lib:${LD_LIBRARY_PATH}"
    },
    cuda=False,
    environment=False,
    gdrcopy="/usr/local/gdrcopy",
    knem="/usr/local/knem",
    prefix="/usr/local/ucx-mlnx-legacy",
    disable_static=True,
    enable_mt=True,
    with_verbs="/usr/local/ofed/4.6-1.0.1.1/usr",
    with_rdmacm="/usr/local/ofed/4.6-1.0.1.1/usr",
)

# Symlink legacy UCX into legacy OFED versions
Stage0 += shell(
    commands=[
        "ln -s /usr/local/ucx-mlnx-legacy/{1}/* /usr/local/ofed/{0}/usr/{1}".format(
            version, directory
        )
        for version in mlnx_versions
        for directory in ["bin", "lib"]
    ]
)

# PMI support
Stage0 += slurm_pmi2(prefix="/usr/local/pmi")
Stage0 += pmix()

# OpenMPI
mpicc = openmpi(
    cuda=False,
    infiniband=False,
    ldconfig=True,
    ucx=True,
    version=local_args.ompi,
    disable_oshmem=True,
    disable_static=True,
    enable_mca_no_build="btl-uct",
    with_slurm=False,
    with_pmi="/usr/local/pmi",
    with_pmix="/usr/local/pmix",
)
toolchain = mpicc.toolchain
Stage0 += mpicc

# OpenMPI expects this program to exist, even if it's not used.
# Default is "ssh : rsh", but that's not installed.
# TODO: /usr/etc/openmpi-mca-params.conf ?
# Stage0 += shell(
#     commands=[
#         "mkdir /mnt/0",
#         "echo 'plm_rsh_agent = false' >> /mnt/0/openmpi-mca-params.conf",
#         "echo 'mpi_warn_on_fork = 0' >> /mnt/0/openmpi-mca-params.conf",
#         "echo 'btl_openib_warn_default_gid_prefix = 0' >> /mnt/0/openmpi-mca-params.conf",
#     ]
# )

Stage0 += label(
    metadata={
        "org.opengeosys.mpi": "openmpi",
        "org.opengeosys.mpi.version": local_args.ompi,
        # "org.opengeosys.ucx": local_args.ucx,
        # "org.opengeosys.slurm": local_args.slurm,
        # "org.opengeosys.pmix": pmix_version,
    }
)

# Allow running MPI as root
Stage1 += environment(
    variables={"OMPI_ALLOW_RUN_AS_ROOT": "1", "OMPI_ALLOW_RUN_AS_ROOT_CONFIRM": "1"}
)

# Entrypoint
Stage1 += shell(
    commands=[
        "curl -o /usr/local/bin/entrypoint.sh https://gitlab.opengeosys.org/ogs/container-maker/-/raw/refactor-cli-to-api/ogscm/recipes/mpi-entrypoint.sh"
    ]
)
Stage1 += runscript(commands=["/usr/local/bin/entrypoint.sh"])

# Performance and compatibility tuning
Stage1 += environment(
    variables={
        "CUDA_CACHE_DISABLE": "1",
        "MELLANOX_VISIBLE_DEVICES": "all",  # enroot
        "OMPI_MCA_pml": "ucx",
        "UCX_TLS": "all",
    }
)

if local_args.mpi_benchmarks:
    Stage0 += generic_autotools(
        build_environment={"CC": "mpicc", "CXX": "mpicxx"},
        # enable_cuda=True,
        prefix="/usr/local/osu",
        url="http://mvapich.cse.ohio-state.edu/download/mvapich/osu-micro-benchmarks-5.6.3.tar.gz",
        # with_cuda="/usr/local/cuda",
    )

    # Copy the OSU Micro-Benchmark binaries into the deployment stage
    Stage1 += copy(_from="0", src="/usr/local/osu", dest="/usr/local/osu")

    # Add the OSU Micro-Benchmarks to the default PATH
    base_path = "/usr/local/osu/libexec/osu-micro-benchmarks"
    Stage1 += environment(
        variables={
            "PATH": "{0}:{0}/mpi/collective:{0}/mpi/one-sided:{0}/mpi/pt2pt:{0}/mpi/startup:$PATH".format(
                base_path
            )
        }
    )
    # Stage0 += osu_benchmarks(toolchain=toolchain)
    # Stage0 += shell(
    #     commands=[
    #         "mkdir -p /usr/local/mpi-examples",
    #         "cd /usr/local/mpi-examples",
    #         "curl -O https://raw.githubusercontent.com/hpc/charliecloud/674b3b4e4ad243be5565f200d8f5fb92b7544480/examples/mpihello/hello.c",
    #         "curl -O https://computing.llnl.gov/tutorials/mpi/samples/C/mpi_bandwidth.c",
    #         "curl -O https://raw.githubusercontent.com/mpitutorial/mpitutorial/gh-pages/tutorials/mpi-send-and-receive/code/ring.c",
    #         "mpicc -o /usr/local/bin/mpi-hello /usr/local/mpi-examples/hello.c",
    #         "mpicc -o /usr/local/bin/mpi-ring /usr/local/mpi-examples/ring.c",
    #         "mpicc -o /usr/local/bin/mpi-bandwidth /usr/local/mpi-examples/mpi_bandwidth.c",
    #     ]
    # )
    # Stage1 += copy(_from="build", src="/usr/local/bin/mpi-*", dest="/usr/local/bin/")

Stage0 += comment(f"--- End {filename} ---")



import subprocess
import sys
import re
import os


class builder(object):
    def __init__(
        self, args, images_out_dir, img_file, definition_file_path, tag, cwd, **kwargs
    ):
        self.__format = kwargs.get("format", "docker")
        self.__args = args
        self.__images_out_dir = images_out_dir
        self.__img_file = img_file
        self.__definition_file_path = definition_file_path
        self.__tag = tag
        self.__cwd = cwd

    def build(self):
        if self.__format == "singularity":
            self.build_singularity()
        else:
            self.build_docker()

    def build_singularity(self):
        sif_file = f"{self.__images_out_dir}/{self.__img_file}.sif"
        subprocess.run(
            f"sudo `which singularity` build --force {sif_file}"
            f"{self.__definition_file_path}",
            shell=True,
        )
        subprocess.run(
            f"sudo chown $USER:$USER {sif_file}",
            shell=True,
        )
        # TODO: adapt this
        exit(0)

    def build_docker(self):
        build_cmd = (
            f"DOCKER_BUILDKIT=1 docker build {self.__args.build_args} "
            f"-t {self.__tag} -f {self.__definition_file_path} ."
        )
        print(f"Running: {build_cmd}")
        subprocess.run(build_cmd, shell=True)
        inspect_out = subprocess.check_output(
            f"docker inspect {self.__tag} | grep Id", shell=True
        ).decode(sys.stdout.encoding)
        image_id = re.search("sha256:(\w*)", inspect_out).group(1)
        image_id_short = image_id[0:12]

        if self.__args.upload:
            subprocess.run(f"docker push {self.__tag}", shell=True)
        if self.__args.sif_file:
            self.image_file = f"{self.__images_out_dir}/{self.__args.sif_file}"
        else:
            self.image_file = (
                f"{self.__images_out_dir}/{self.__img_file}-{image_id_short}.sif"
            )
        if self.__args.convert and not os.path.exists(self.image_file):
            subprocess.run(
                f"cd {self.__cwd} && singularity build --force {self.image_file} docker-daemon:{self.__tag}",
                shell=True,
            )



# pylint: disable=invalid-name, too-few-public-methods
# pylint: disable=too-many-instance-attributes
"""Container info"""

from __future__ import absolute_import
from __future__ import unicode_literals
from __future__ import print_function

import hashlib
import json
import os
import re
import requests
import shutil
import subprocess

from ogscm import config


class container_info:
    def __init__(self, args):
        """Initialize container info"""
        self.branch = ""
        self.commit_hash = ""
        self.cwd = os.getcwd()
        self.definition_file = ""
        self.definition_file_path = ""
        self.git_version = ""
        self.images_out_dir = ""
        self.img_file = ""
        self.ogsdir = False
        self.outdir = ""
        self.repo = ""
        self.scif_file = ""

        name_start = "gcc"
        branch_is_release = False

        if args.ogs != "off" and args.ogs != "clean":
            if os.path.isdir(args.ogs):
                self.repo = "local"
                self.commit_hash = subprocess.run(
                    ["cd {} && git rev-parse HEAD".format(args.ogs)],
                    capture_output=True,
                    text=True,
                    shell=True,
                ).stdout.rstrip()
                if "GITLAB_CI" in os.environ:
                    if "CI_COMMIT_BRANCH" in os.environ:
                        self.branch = os.environ["CI_COMMIT_BRANCH"]
                    elif "CI_MERGE_REQUEST_SOURCE_BRANCH_NAME" in os.environ:
                        self.branch = os.environ["CI_MERGE_REQUEST_SOURCE_BRANCH_NAME"]
                    self.git_version = os.getenv("args.ogs", "x.x.x")
                else:
                    self.branch = subprocess.run(
                        [
                            "cd {} && git branch | grep \* | cut -d ' ' -f2".format(
                                args.ogs
                            )
                        ],
                        capture_output=True,
                        text=True,
                        shell=True,
                    ).stdout
                    self.git_version = subprocess.run(
                        ["cd {} && git describe --tags".format(args.ogs)],
                        capture_output=True,
                        text=True,
                        shell=True,
                    ).stdout[0]
            else:
                # Get git commit hash and construct image tag name
                self.repo, self.branch, *commit = args.ogs.split("@")
                if commit:
                    self.commit_hash = commit[0]
                    if self.branch == "":
                        self.branch = "master"
                else:
                    if re.search(r"[\d.]+", self.branch):
                        branch_is_release = True
                    repo_split = self.repo.split("/")
                    response = requests.get(
                        f"https://gitlab.opengeosys.org/api/v4/projects/{self.repo.replace('/', '%2F')}/repository/commits?ref_name={self.branch}"
                    )
                    response_data = json.loads(response.text)
                    self.commit_hash = response_data[0]["id"]
                    # ogs_tag = args.ogs.replace('/', '.').replace('@', '.')

            if branch_is_release:
                name_start = f"ogs-{self.branch}"
            else:
                name_start = f"ogs-{self.commit_hash[:8]}"
        else:
            if args.compiler == "clang":
                name_start = "clang"

        name_openmpi = "serial"
        if args.ompi != "off":
            name_openmpi = f"openmpi-{args.ompi}"

        if len(args.cmake_args) > 0:
            cmake_args_hash = hashlib.md5(
                " ".join(args.cmake_args).encode("utf-8")
            ).hexdigest()
            cmake_args_hash_short = cmake_args_hash[:8]

        # name_image = args.base_image.replace(':', '_')
        # Removed {name_image}/
        img_folder = (
            f"{name_start}/{name_openmpi}/" f"{config.g_package_manager.name.lower()}"
        )
        self.img_file = img_folder.replace("/", "-")
        if len(args.cmake_args) > 0:
            self.img_file += f"-cmake-{cmake_args_hash_short}"
        if args.gui:
            self.img_file += "-gui"
        if args.ogs != "off" and not args.runtime_only:
            self.img_file += "-dev"

        if args.tag != "":
            self.tag = args.tag
        else:
            self.tag = f"{args.registry}/{self.img_file}:latest"

        if os.path.isdir(args.ogs):
            self.ogsdir = True

        if args.file != "":
            self.out_dir = args.out
            self.definition_file = args.file
        else:
            if self.ogsdir:
                self.out_dir = os.path.join(
                    args.ogs, f"{args.out}/{args.format}/{img_folder}"
                )
            else:
                self.out_dir = f"{args.out}/{args.format}/{img_folder}"
            if len(args.cmake_args) > 0:
                self.out_dir += f"/cmake-{cmake_args_hash_short}"
            self.images_out_dir = f"{args.out}/images"
            self.definition_file = "Dockerfile"
            if args.format == "singularity":
                self.definition_file = "Singularity.def"

        if self.ogsdir:
            context_path_size = len(self.ogsdir)
            scif_file = f"{self.out_dir[context_path_size+1:]}/ogs.scif"
            self.definition_file_path = (
                f"{self.out_dir[context_path_size+1:]}/{self.definition_file}"
            )
        else:
            self.scif_file = f"{self.out_dir}/ogs.scif"
            self.definition_file_path = os.path.join(self.out_dir, self.definition_file)

    def make_dirs(self):
        if not os.path.exists(self.out_dir):
            os.makedirs(self.out_dir)  # For .scif files
        if self.images_out_dir and not os.path.exists(self.images_out_dir):
            os.makedirs(self.images_out_dir)

    def cleanup(self):
        shutil.rmtree(self.out_dir, ignore_errors=True)
        print("Cleaned up!")



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['ogscm', 'ogscm.app', 'ogscm.building_blocks', 'ogscm.recipes']

package_data = \
{'': ['*']}

install_requires = \
['PyYAML>=5.3.1,<6.0.0',
 'hpccm>=20.10.0,<21.0.0',
 'packaging>=20.4,<21.0',
 'requests>=2.24.0,<3.0.0']

entry_points = \
{'console_scripts': ['ogscm = ogscm.cli:main']}

setup_kwargs = {
    'name': 'ogscm',
    'version': '2.0.0',
    'description': 'OGS Container Maker',
    'long_description': "# OGS Container Maker\n\n## General usage\n\n### Installation\n\n```bash\nvirtualenv ~/.venv/ogs-container-maker\nsource ~/.venv/ogs-container-maker/bin/activate\npip install ogscm\n```\n\n### Generate container definition\n\nOGS Container Maker has builtin recipes. You need to specify the recipes to use as command arguments. Each of recipes adds options to the tool. Typically you want to start with a compiler. Add the `compiler.py` recipe and the `--help`-flag to get more options:\n\n```bash\n$ ogscm compiler.py --help\n...\ncompiler.py:\n  --compiler COMPILER   The compiler to use. Possible options: off, gcc,\n                        clang (default: gcc)\n  --compiler_version COMPILER_VERSION\n                        Compiler version. (default: )\n  --iwyy                Install include-what-you-use (requires clang\n                        compiler) (default: False)\n```\n\nAfter specifying the compiler recipe (and optionally setting a non-default compiler and version) you may want to add the `ogs.py` recipe:\n\n```bash\n$ ogscm compiler.py ogs.py --help\n...\nogs.py:\n  --pm {system,conan,off}\n                        Package manager to install third-party dependencies\n                        (default: conan)\n  --ogs OGS             OGS repo on gitlab.opengeosys.org in the form\n                        'user/repo@branch' OR 'user/repo@@commit' to\n                        checkout a specific commit OR a path to a local\n                        subdirectory to the git cloned OGS sources OR 'off'\n                        to disable OGS building OR 'clean' to disable OGS\n                        and all its dev dependencies (default:\n                        ogs/ogs@master)\n  --cmake_args CMAKE_ARGS\n                        CMake argument set has to be quoted and **must**\n                        start with a space. e.g. --cmake_args '\n                        -DFIRST=TRUE -DFOO=BAR' (default: )\n...\n```\n\nTo generate a Dockerfile with the default parameters:\n\n```bash\n$ ogscm compiler.py ogs.py\nEvaluating compiler.py\nEvaluating ogs.py\nCreated definition _out/docker/gcc/default/ogs-d18c786e/conan/Dockerfile\n```\n\nWith some options (and the `mpi.py`-recipe):\n\n```bash\n$ ogscm compiler.py mpi.py ogs.py --base_image 'centos:8' --ompi 4.0.5 --cmake_args ' -DOGS_BUILD_PROCESSES=LiquidFlow'\nEvaluating compiler.py\nEvaluating mpi.py\nEvaluating ogs.py\nCreated definition _out/docker/gcc/10/openmpi/4.0.5/ogs-d18c786e/conan/cmake-702517b3/Dockerfile\n```\n\n### Build image\n\nAdd the `--build`-flag.\n\nConvert Docker image to Singularity image:\n\nAdd the `--convert`-flag (requires Singularity 3.x).\n\n### Run\n\n```bash\ndocker run --it --rm ogs-ompi-2.1.3\n# in container:\nogs --version\n```\n\n```bash\nsingularity shell ogs-ompi-2.1.3.sif\n# in container:\nogs --version\n# OR directly run from host\nsingularity exec ogs-ompi-2.1.3.sif ogs local/path/to/square_1e0.prj\n```\n\n## All options\n\nAll options for current builtin recipes:\n\n```\n$ ogscm compiler.py mpi.py ogs.py --base_image 'centos:8' --help\nEvaluating compiler.py\nEvaluating mpi.py\nEvaluating ogs.py\nusage: ogscm [-h] [--version] [--out OUT] [--file FILE] [--sif_file SIF_FILE] [--print] [--format {docker,singularity}]\n             [--base_image BASE_IMAGE] [--build] [--build_args BUILD_ARGS] [--upload] [--registry REGISTRY] [--tag TAG]\n             [--convert] [--runtime-only] [--clean] [--deploy [DEPLOY]] [--pip [package [package ...]]]\n             [--packages [packages [packages ...]]] [--compiler COMPILER] [--compiler_version COMPILER_VERSION]\n             [--iwyy] [--ompi OMPI] [--mpi_benchmarks] [--pm {system,conan,off}] [--ogs OGS] [--cmake_args CMAKE_ARGS]\n             [--ccache] [--parallel PARALLEL] [--gui] [--docs] [--cvode] [--cppcheck] [--gcovr] [--tfel] [--insitu]\n             [--dev]\n             recipe [recipe ...]\n\npositional arguments:\n  recipe\n\noptional arguments:\n  -h, --help            show this help message and exit\n  --version             show program's version number and exit\n  --out OUT             Output directory (default: _out)\n  --file FILE           Overwrite output recipe file name (default: )\n  --sif_file SIF_FILE   Overwrite output singularity image file name (default: )\n  --print, -P           Print the definition to stdout (default: False)\n\nGeneral image config:\n  --format {docker,singularity}\n  --base_image BASE_IMAGE\n                        The base image. (centos:8 is supported too) (default: ubuntu:20.04)\n\nImage build options:\n  --build, -B           Build the images from the definition files (default: False)\n  --build_args BUILD_ARGS\n                        Arguments to the build command. Have to be quoted and **must** start with a space. E.g.\n                        --build_args ' --no-cache' (default: )\n  --upload, -U          Upload Docker image to registry (default: False)\n  --registry REGISTRY   The docker registry the image is tagged and uploaded to. (default:\n                        registry.opengeosys.org/ogs/ogs)\n  --tag TAG             The full docker image tag. Overwrites --registry. (default: )\n  --convert, -C         Convert Docker image to Singularity image (default: False)\n  --runtime-only, -R    Generate multi-stage Dockerfiles for small runtime images (default: False)\n\nMaintenance:\n  --clean               Cleans up generated files in default directories. (default: False)\n\nImage deployment:\n  --deploy [DEPLOY], -D [DEPLOY]\n                        Deploys to all configured hosts (in config/deploy_hosts.yml) with no additional arguments or to\n                        the specified host. Implies --build and --convert arguments. (default: )\n\nPackages to install:\n  --pip [package [package ...]]\n                        Install additional Python packages (default: [])\n  --packages [packages [packages ...]]\n                        Install additional OS packages (default: [])\n\ncompiler.py:\n  --compiler COMPILER   The compiler to use. Possible options: off, gcc, clang (default: gcc)\n  --compiler_version COMPILER_VERSION\n                        Compiler version. (default: )\n  --iwyy                Install include-what-you-use (requires clang compiler) (default: False)\n\nmpi.py:\n  --ompi OMPI           OpenMPI version, e.g. 2.1.1, 2.1.5, 3.0.1, 3.1.2 (default: 4.0.5)\n  --mpi_benchmarks      Installs OSU MPI benchmarks as scif app and mpi_bw, mpi_ring,mpi_hello (default: False)\n\nogs.py:\n  --pm {system,conan,off}\n                        Package manager to install third-party dependencies (default: conan)\n  --ogs OGS             OGS repo on gitlab.opengeosys.org in the form 'user/repo@branch' OR 'user/repo@@commit' to\n                        checkout a specific commit OR a path to a local subdirectory to the git cloned OGS sources OR\n                        'off' to disable OGS building OR 'clean' to disable OGS and all its dev dependencies (default:\n                        ogs/ogs@master)\n  --cmake_args CMAKE_ARGS\n                        CMake argument set has to be quoted and **must** start with a space. e.g. --cmake_args '\n                        -DFIRST=TRUE -DFOO=BAR' (default: )\n  --ccache              Enables ccache build caching. (default: False)\n  --parallel PARALLEL, -j PARALLEL\n                        The number of cores to use for compilation. (default: 4)\n  --gui                 Builds the GUI (Data Explorer) (default: False)\n  --docs                Setup documentation requirements (Doxygen) (default: False)\n  --cvode               Install and configure with cvode (default: False)\n  --cppcheck            Install cppcheck (default: False)\n  --gcovr               Install gcovr (default: False)\n  --tfel                Install tfel (default: False)\n  --insitu              Builds with insitu capabilities (default: False)\n  --dev                 Installs development tools (vim, gdb) (default: False)\n```\n\n## Advanced usage\n\n### Build OGS from local git repo\n\nYou can use the ogs-container-maker to build multiple container images from your current source code at once. The following commands will build (`-B`-parameter) 4 docker container (using one serial and 3 MPI-enabled configurations), convert them to Singularity image files (`-C`) and strip everything out but the runtime-requirements (`-R`). You can find the images in `_out/images`.\n\n```\nvirtualenv .venv\nsource .venv/bin/activate\npip install ogscm\nogscm compiler.py ogs.py -B -C -R --ogs [path to ogs sources] --pm system\n```\n\n### Deploy image files\n\n- Requires `rsync`\n- Rename the file `config/deploy_hosts_example.yml` to `config/deploy_hosts.yml`\n- `host` has to be a SSH host to which you have passwordless access\n- Deploy to the host with `... -D myhost`\n\n\n## PyPi Publication\n\n- Bump version in `pyproject.py` and `version.py`\n- Create tag\n- Push to GitLab (`git push --tags`)\n",
    'author': 'Lars Bilke',
    'author_email': 'lars.bilke@ufz.de',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://gitlab.opengeosys.org/ogs/container-maker',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
